/* SPDX-License-Identifier: GPL-2.0 WITH Linux-syscall-note */
/*
 *
 * (C) COPYRIGHT 2014-2024 ARM Limited. All rights reserved.
 *
 * This program is free software and is provided to you under the terms of the
 * GNU General Public License version 2 as published by the Free Software
 * Foundation, and any use by you of this program is subject to the terms
 * of such GNU license.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, you can access it online at
 * http://www.gnu.org/licenses/gpl-2.0.html.
 *
 */

#ifndef _KBASE_HWCONFIG_ISSUES_H_
#define _KBASE_HWCONFIG_ISSUES_H_

#include <linux/version_compat_defs.h>

enum base_hw_issue {
	KBASE_HW_ISSUE_5736,
	KBASE_HW_ISSUE_9435,
	KBASE_HW_ISSUE_10682,
	KBASE_HW_ISSUE_11054,
	KBASE_HW_ISSUE_T76X_3953,
	KBASE_HW_ISSUE_TMIX_7891,
	KBASE_HW_ISSUE_TMIX_7940,
	KBASE_HW_ISSUE_TMIX_8042,
	KBASE_HW_ISSUE_TMIX_8133,
	KBASE_HW_ISSUE_TMIX_8138,
	KBASE_HW_ISSUE_TMIX_8206,
	KBASE_HW_ISSUE_TMIX_8343,
	KBASE_HW_ISSUE_TMIX_8463,
	KBASE_HW_ISSUE_TMIX_8456,
	KBASE_HW_ISSUE_TSIX_1116,
	KBASE_HW_ISSUE_TSIX_2033,
	KBASE_HW_ISSUE_TMIX_8438,
	KBASE_HW_ISSUE_TNOX_1194,
	KBASE_HW_ISSUE_TGOX_R1_1234,
	KBASE_HW_ISSUE_TTRX_1337,
	KBASE_HW_ISSUE_TSIX_1792,
	KBASE_HW_ISSUE_TTRX_2968_TTRX_3162,
	KBASE_HW_ISSUE_TTRX_3076,
	KBASE_HW_ISSUE_TTRX_921,
	KBASE_HW_ISSUE_TTRX_3414,
	KBASE_HW_ISSUE_GPU2017_1336,
	KBASE_HW_ISSUE_TTRX_3083,
	KBASE_HW_ISSUE_TTRX_3470,
	KBASE_HW_ISSUE_TTRX_3464,
	KBASE_HW_ISSUE_TTRX_3485,
	KBASE_HW_ISSUE_GPU2019_3212,
	KBASE_HW_ISSUE_TURSEHW_1997,
	KBASE_HW_ISSUE_GPU2019_3878,
	KBASE_HW_ISSUE_TURSEHW_2716,
	KBASE_HW_ISSUE_GPU2019_3901,
	KBASE_HW_ISSUE_GPU2021PRO_290,
	KBASE_HW_ISSUE_TITANHW_2710,
	KBASE_HW_ISSUE_TITANHW_2679,
	KBASE_HW_ISSUE_GPU2022PRO_148,
	KBASE_HW_ISSUE_TITANHW_2922,
	KBASE_HW_ISSUE_TITANHW_2952,
	KBASE_HW_ISSUE_KRAKEHW_2151,
	KBASE_HW_ISSUE_TITANHW_2938,
	KBASE_HW_ISSUE_KRAKEHW_2269,
	KBASE_HW_ISSUE_TURSEHW_2934,
	KBASE_HW_ISSUE_KRAKEHW_2321,
	KBASE_HW_ISSUE_END
};

__maybe_unused static const enum base_hw_issue base_hw_issues_generic[] = { KBASE_HW_ISSUE_END };

__maybe_unused static const enum base_hw_issue base_hw_issues_tMIx_r0p0_05dev0[] = {
	KBASE_HW_ISSUE_9435,	     KBASE_HW_ISSUE_10682,	  KBASE_HW_ISSUE_11054,
	KBASE_HW_ISSUE_T76X_3953,    KBASE_HW_ISSUE_TMIX_7891,	  KBASE_HW_ISSUE_TMIX_8042,
	KBASE_HW_ISSUE_TMIX_8133,    KBASE_HW_ISSUE_TMIX_8138,	  KBASE_HW_ISSUE_TMIX_8206,
	KBASE_HW_ISSUE_TMIX_8343,    KBASE_HW_ISSUE_TMIX_8463,	  KBASE_HW_ISSUE_TMIX_8456,
	KBASE_HW_ISSUE_TMIX_8438,    KBASE_HW_ISSUE_TSIX_2033,	  KBASE_HW_ISSUE_TTRX_921,
	KBASE_HW_ISSUE_GPU2017_1336, KBASE_HW_ISSUE_TITANHW_2710, KBASE_HW_ISSUE_GPU2022PRO_148,
	KBASE_HW_ISSUE_TITANHW_2938, KBASE_HW_ISSUE_END
};

__maybe_unused static const enum base_hw_issue base_hw_issues_tMIx_r0p0[] = {
	KBASE_HW_ISSUE_9435,	     KBASE_HW_ISSUE_10682,	  KBASE_HW_ISSUE_11054,
	KBASE_HW_ISSUE_TMIX_7891,    KBASE_HW_ISSUE_TMIX_7940,	  KBASE_HW_ISSUE_TMIX_8042,
	KBASE_HW_ISSUE_TMIX_8133,    KBASE_HW_ISSUE_TMIX_8138,	  KBASE_HW_ISSUE_TMIX_8206,
	KBASE_HW_ISSUE_TMIX_8343,    KBASE_HW_ISSUE_TMIX_8463,	  KBASE_HW_ISSUE_TMIX_8456,
	KBASE_HW_ISSUE_TMIX_8438,    KBASE_HW_ISSUE_TSIX_2033,	  KBASE_HW_ISSUE_TTRX_921,
	KBASE_HW_ISSUE_GPU2017_1336, KBASE_HW_ISSUE_TITANHW_2710, KBASE_HW_ISSUE_GPU2022PRO_148,
	KBASE_HW_ISSUE_TITANHW_2938, KBASE_HW_ISSUE_END
};

__maybe_unused static const enum base_hw_issue base_hw_issues_tMIx_r0p1[] = {
	KBASE_HW_ISSUE_9435,	     KBASE_HW_ISSUE_10682,	  KBASE_HW_ISSUE_11054,
	KBASE_HW_ISSUE_TMIX_7891,    KBASE_HW_ISSUE_TMIX_7940,	  KBASE_HW_ISSUE_TMIX_8042,
	KBASE_HW_ISSUE_TMIX_8133,    KBASE_HW_ISSUE_TMIX_8138,	  KBASE_HW_ISSUE_TMIX_8206,
	KBASE_HW_ISSUE_TMIX_8343,    KBASE_HW_ISSUE_TMIX_8463,	  KBASE_HW_ISSUE_TMIX_8456,
	KBASE_HW_ISSUE_TMIX_8438,    KBASE_HW_ISSUE_TSIX_2033,	  KBASE_HW_ISSUE_TTRX_921,
	KBASE_HW_ISSUE_GPU2017_1336, KBASE_HW_ISSUE_TITANHW_2710, KBASE_HW_ISSUE_GPU2022PRO_148,
	KBASE_HW_ISSUE_TITANHW_2938, KBASE_HW_ISSUE_END
};

__maybe_unused static const enum base_hw_issue base_hw_issues_model_tMIx[] = {
	KBASE_HW_ISSUE_5736,	       KBASE_HW_ISSUE_9435,	    KBASE_HW_ISSUE_TMIX_7891,
	KBASE_HW_ISSUE_TMIX_7940,      KBASE_HW_ISSUE_TMIX_8042,    KBASE_HW_ISSUE_TMIX_8133,
	KBASE_HW_ISSUE_TMIX_8138,      KBASE_HW_ISSUE_TMIX_8206,    KBASE_HW_ISSUE_TMIX_8343,
	KBASE_HW_ISSUE_TMIX_8456,      KBASE_HW_ISSUE_TSIX_2033,    KBASE_HW_ISSUE_TITANHW_2710,
	KBASE_HW_ISSUE_GPU2022PRO_148, KBASE_HW_ISSUE_TITANHW_2938, KBASE_HW_ISSUE_END
};

__maybe_unused static const enum base_hw_issue base_hw_issues_tHEx_r0p0[] = {
	KBASE_HW_ISSUE_9435,	     KBASE_HW_ISSUE_10682,	    KBASE_HW_ISSUE_11054,
	KBASE_HW_ISSUE_TMIX_7891,    KBASE_HW_ISSUE_TMIX_8042,	    KBASE_HW_ISSUE_TMIX_8133,
	KBASE_HW_ISSUE_TSIX_2033,    KBASE_HW_ISSUE_TTRX_921,	    KBASE_HW_ISSUE_GPU2017_1336,
	KBASE_HW_ISSUE_TITANHW_2710, KBASE_HW_ISSUE_GPU2022PRO_148, KBASE_HW_ISSUE_TITANHW_2938,
	KBASE_HW_ISSUE_END
};

__maybe_unused static const enum base_hw_issue base_hw_issues_tHEx_r0p1[] = {
	KBASE_HW_ISSUE_9435,	     KBASE_HW_ISSUE_10682,	    KBASE_HW_ISSUE_11054,
	KBASE_HW_ISSUE_TMIX_7891,    KBASE_HW_ISSUE_TMIX_8042,	    KBASE_HW_ISSUE_TMIX_8133,
	KBASE_HW_ISSUE_TSIX_2033,    KBASE_HW_ISSUE_TTRX_921,	    KBASE_HW_ISSUE_GPU2017_1336,
	KBASE_HW_ISSUE_TITANHW_2710, KBASE_HW_ISSUE_GPU2022PRO_148, KBASE_HW_ISSUE_TITANHW_2938,
	KBASE_HW_ISSUE_END
};

__maybe_unused static const enum base_hw_issue base_hw_issues_tHEx_r0p2[] = {
	KBASE_HW_ISSUE_9435,	     KBASE_HW_ISSUE_10682,	    KBASE_HW_ISSUE_11054,
	KBASE_HW_ISSUE_TMIX_7891,    KBASE_HW_ISSUE_TMIX_8042,	    KBASE_HW_ISSUE_TMIX_8133,
	KBASE_HW_ISSUE_TSIX_2033,    KBASE_HW_ISSUE_TTRX_921,	    KBASE_HW_ISSUE_GPU2017_1336,
	KBASE_HW_ISSUE_TITANHW_2710, KBASE_HW_ISSUE_GPU2022PRO_148, KBASE_HW_ISSUE_TITANHW_2938,
	KBASE_HW_ISSUE_END
};

__maybe_unused static const enum base_hw_issue base_hw_issues_tHEx_r0p3[] = {
	KBASE_HW_ISSUE_9435,	       KBASE_HW_ISSUE_10682,	    KBASE_HW_ISSUE_TMIX_7891,
	KBASE_HW_ISSUE_TMIX_8042,      KBASE_HW_ISSUE_TMIX_8133,    KBASE_HW_ISSUE_TSIX_2033,
	KBASE_HW_ISSUE_TTRX_921,       KBASE_HW_ISSUE_GPU2017_1336, KBASE_HW_ISSUE_TITANHW_2710,
	KBASE_HW_ISSUE_GPU2022PRO_148, KBASE_HW_ISSUE_TITANHW_2938, KBASE_HW_ISSUE_END
};

__maybe_unused static const enum base_hw_issue base_hw_issues_model_tHEx[] = {
	KBASE_HW_ISSUE_5736,	     KBASE_HW_ISSUE_9435,	    KBASE_HW_ISSUE_TMIX_7891,
	KBASE_HW_ISSUE_TMIX_8042,    KBASE_HW_ISSUE_TMIX_8133,	    KBASE_HW_ISSUE_TSIX_2033,
	KBASE_HW_ISSUE_TITANHW_2710, KBASE_HW_ISSUE_GPU2022PRO_148, KBASE_HW_ISSUE_TITANHW_2938,
	KBASE_HW_ISSUE_END
};

__maybe_unused static const enum base_hw_issue base_hw_issues_tSIx_r0p0[] = {
	KBASE_HW_ISSUE_9435,	     KBASE_HW_ISSUE_11054,	    KBASE_HW_ISSUE_TMIX_8133,
	KBASE_HW_ISSUE_TSIX_1116,    KBASE_HW_ISSUE_TSIX_2033,	    KBASE_HW_ISSUE_TSIX_1792,
	KBASE_HW_ISSUE_TTRX_921,     KBASE_HW_ISSUE_GPU2017_1336,   KBASE_HW_ISSUE_TTRX_3464,
	KBASE_HW_ISSUE_TITANHW_2710, KBASE_HW_ISSUE_GPU2022PRO_148, KBASE_HW_ISSUE_TITANHW_2938,
	KBASE_HW_ISSUE_END
};

__maybe_unused static const enum base_hw_issue base_hw_issues_tSIx_r0p1[] = {
	KBASE_HW_ISSUE_9435,	     KBASE_HW_ISSUE_11054,	    KBASE_HW_ISSUE_TMIX_8133,
	KBASE_HW_ISSUE_TSIX_1116,    KBASE_HW_ISSUE_TSIX_2033,	    KBASE_HW_ISSUE_TSIX_1792,
	KBASE_HW_ISSUE_TTRX_921,     KBASE_HW_ISSUE_GPU2017_1336,   KBASE_HW_ISSUE_TTRX_3464,
	KBASE_HW_ISSUE_TITANHW_2710, KBASE_HW_ISSUE_GPU2022PRO_148, KBASE_HW_ISSUE_TITANHW_2938,
	KBASE_HW_ISSUE_END
};

__maybe_unused static const enum base_hw_issue base_hw_issues_tSIx_r1p0[] = {
	KBASE_HW_ISSUE_9435,	       KBASE_HW_ISSUE_11054,	    KBASE_HW_ISSUE_TMIX_8133,
	KBASE_HW_ISSUE_TSIX_1116,      KBASE_HW_ISSUE_TSIX_2033,    KBASE_HW_ISSUE_TTRX_921,
	KBASE_HW_ISSUE_GPU2017_1336,   KBASE_HW_ISSUE_TTRX_3464,    KBASE_HW_ISSUE_TITANHW_2710,
	KBASE_HW_ISSUE_GPU2022PRO_148, KBASE_HW_ISSUE_TITANHW_2938, KBASE_HW_ISSUE_END
};

__maybe_unused static const enum base_hw_issue base_hw_issues_tSIx_r1p1[] = {
	KBASE_HW_ISSUE_9435,	     KBASE_HW_ISSUE_TMIX_8133,	  KBASE_HW_ISSUE_TSIX_1116,
	KBASE_HW_ISSUE_TSIX_2033,    KBASE_HW_ISSUE_TTRX_921,	  KBASE_HW_ISSUE_GPU2017_1336,
	KBASE_HW_ISSUE_TTRX_3464,    KBASE_HW_ISSUE_TITANHW_2710, KBASE_HW_ISSUE_GPU2022PRO_148,
	KBASE_HW_ISSUE_TITANHW_2938, KBASE_HW_ISSUE_END
};

__maybe_unused static const enum base_hw_issue base_hw_issues_model_tSIx[] = {
	KBASE_HW_ISSUE_5736,	     KBASE_HW_ISSUE_9435,	    KBASE_HW_ISSUE_TMIX_8133,
	KBASE_HW_ISSUE_TSIX_1116,    KBASE_HW_ISSUE_TSIX_2033,	    KBASE_HW_ISSUE_TTRX_3464,
	KBASE_HW_ISSUE_TITANHW_2710, KBASE_HW_ISSUE_GPU2022PRO_148, KBASE_HW_ISSUE_TITANHW_2938,
	KBASE_HW_ISSUE_END
};

__maybe_unused static const enum base_hw_issue base_hw_issues_tDVx_r0p0[] = {
	KBASE_HW_ISSUE_9435,	     KBASE_HW_ISSUE_TMIX_8133,	  KBASE_HW_ISSUE_TSIX_1116,
	KBASE_HW_ISSUE_TSIX_2033,    KBASE_HW_ISSUE_TTRX_921,	  KBASE_HW_ISSUE_GPU2017_1336,
	KBASE_HW_ISSUE_TTRX_3464,    KBASE_HW_ISSUE_TITANHW_2710, KBASE_HW_ISSUE_GPU2022PRO_148,
	KBASE_HW_ISSUE_TITANHW_2938, KBASE_HW_ISSUE_END
};

__maybe_unused static const enum base_hw_issue base_hw_issues_model_tDVx[] = {
	KBASE_HW_ISSUE_5736,	     KBASE_HW_ISSUE_9435,	    KBASE_HW_ISSUE_TMIX_8133,
	KBASE_HW_ISSUE_TSIX_1116,    KBASE_HW_ISSUE_TSIX_2033,	    KBASE_HW_ISSUE_TTRX_3464,
	KBASE_HW_ISSUE_TITANHW_2710, KBASE_HW_ISSUE_GPU2022PRO_148, KBASE_HW_ISSUE_TITANHW_2938,
	KBASE_HW_ISSUE_END
};

__maybe_unused static const enum base_hw_issue base_hw_issues_tNOx_r0p0[] = {
	KBASE_HW_ISSUE_9435,	     KBASE_HW_ISSUE_TMIX_8133,
	KBASE_HW_ISSUE_TSIX_1116,    KBASE_HW_ISSUE_TSIX_2033,
	KBASE_HW_ISSUE_TNOX_1194,    KBASE_HW_ISSUE_TTRX_921,
	KBASE_HW_ISSUE_GPU2017_1336, KBASE_HW_ISSUE_TTRX_3464,
	KBASE_HW_ISSUE_TITANHW_2710, KBASE_HW_ISSUE_GPU2022PRO_148,
	KBASE_HW_ISSUE_TITANHW_2938, KBASE_HW_ISSUE_KRAKEHW_2321,
	KBASE_HW_ISSUE_END
};

__maybe_unused static const enum base_hw_issue base_hw_issues_model_tNOx[] = {
	KBASE_HW_ISSUE_5736,	     KBASE_HW_ISSUE_9435,
	KBASE_HW_ISSUE_TMIX_8133,    KBASE_HW_ISSUE_TSIX_1116,
	KBASE_HW_ISSUE_TSIX_2033,    KBASE_HW_ISSUE_TTRX_3464,
	KBASE_HW_ISSUE_TITANHW_2710, KBASE_HW_ISSUE_GPU2022PRO_148,
	KBASE_HW_ISSUE_TITANHW_2938, KBASE_HW_ISSUE_KRAKEHW_2321,
	KBASE_HW_ISSUE_END
};

__maybe_unused static const enum base_hw_issue base_hw_issues_tGOx_r0p0[] = {
	KBASE_HW_ISSUE_9435,	     KBASE_HW_ISSUE_TMIX_8133,
	KBASE_HW_ISSUE_TSIX_1116,    KBASE_HW_ISSUE_TSIX_2033,
	KBASE_HW_ISSUE_TNOX_1194,    KBASE_HW_ISSUE_TTRX_921,
	KBASE_HW_ISSUE_GPU2017_1336, KBASE_HW_ISSUE_TTRX_3464,
	KBASE_HW_ISSUE_TITANHW_2710, KBASE_HW_ISSUE_GPU2022PRO_148,
	KBASE_HW_ISSUE_TITANHW_2938, KBASE_HW_ISSUE_KRAKEHW_2321,
	KBASE_HW_ISSUE_END
};

__maybe_unused static const enum base_hw_issue base_hw_issues_tGOx_r1p0[] = {
	KBASE_HW_ISSUE_9435,	     KBASE_HW_ISSUE_TMIX_8133,
	KBASE_HW_ISSUE_TSIX_1116,    KBASE_HW_ISSUE_TSIX_2033,
	KBASE_HW_ISSUE_TGOX_R1_1234, KBASE_HW_ISSUE_TTRX_921,
	KBASE_HW_ISSUE_GPU2017_1336, KBASE_HW_ISSUE_TTRX_3464,
	KBASE_HW_ISSUE_TITANHW_2710, KBASE_HW_ISSUE_GPU2022PRO_148,
	KBASE_HW_ISSUE_TITANHW_2938, KBASE_HW_ISSUE_KRAKEHW_2321,
	KBASE_HW_ISSUE_END
};

__maybe_unused static const enum base_hw_issue base_hw_issues_model_tGOx[] = {
	KBASE_HW_ISSUE_5736,	     KBASE_HW_ISSUE_9435,
	KBASE_HW_ISSUE_TMIX_8133,    KBASE_HW_ISSUE_TSIX_1116,
	KBASE_HW_ISSUE_TSIX_2033,    KBASE_HW_ISSUE_TTRX_3464,
	KBASE_HW_ISSUE_TITANHW_2710, KBASE_HW_ISSUE_GPU2022PRO_148,
	KBASE_HW_ISSUE_TITANHW_2938, KBASE_HW_ISSUE_KRAKEHW_2321,
	KBASE_HW_ISSUE_END
};

__maybe_unused static const enum base_hw_issue base_hw_issues_tTRx_r0p0[] = {
	KBASE_HW_ISSUE_9435,	     KBASE_HW_ISSUE_TSIX_2033,
	KBASE_HW_ISSUE_TTRX_1337,    KBASE_HW_ISSUE_TTRX_2968_TTRX_3162,
	KBASE_HW_ISSUE_TTRX_3076,    KBASE_HW_ISSUE_TTRX_921,
	KBASE_HW_ISSUE_TTRX_3414,    KBASE_HW_ISSUE_GPU2017_1336,
	KBASE_HW_ISSUE_TTRX_3083,    KBASE_HW_ISSUE_TTRX_3470,
	KBASE_HW_ISSUE_TTRX_3464,    KBASE_HW_ISSUE_TTRX_3485,
	KBASE_HW_ISSUE_TITANHW_2710, KBASE_HW_ISSUE_GPU2022PRO_148,
	KBASE_HW_ISSUE_TITANHW_2938, KBASE_HW_ISSUE_END
};

__maybe_unused static const enum base_hw_issue base_hw_issues_tTRx_r0p1[] = {
	KBASE_HW_ISSUE_9435,	     KBASE_HW_ISSUE_TSIX_2033,
	KBASE_HW_ISSUE_TTRX_1337,    KBASE_HW_ISSUE_TTRX_2968_TTRX_3162,
	KBASE_HW_ISSUE_TTRX_3076,    KBASE_HW_ISSUE_TTRX_921,
	KBASE_HW_ISSUE_TTRX_3414,    KBASE_HW_ISSUE_GPU2017_1336,
	KBASE_HW_ISSUE_TTRX_3083,    KBASE_HW_ISSUE_TTRX_3470,
	KBASE_HW_ISSUE_TTRX_3464,    KBASE_HW_ISSUE_TTRX_3485,
	KBASE_HW_ISSUE_TITANHW_2710, KBASE_HW_ISSUE_GPU2022PRO_148,
	KBASE_HW_ISSUE_TITANHW_2938, KBASE_HW_ISSUE_END
};

__maybe_unused static const enum base_hw_issue base_hw_issues_tTRx_r0p2[] = {
	KBASE_HW_ISSUE_9435,
	KBASE_HW_ISSUE_TSIX_2033,
	KBASE_HW_ISSUE_TTRX_1337,
	KBASE_HW_ISSUE_TTRX_2968_TTRX_3162,
	KBASE_HW_ISSUE_TTRX_3076,
	KBASE_HW_ISSUE_TTRX_921,
	KBASE_HW_ISSUE_TTRX_3414,
	KBASE_HW_ISSUE_GPU2017_1336,
	KBASE_HW_ISSUE_TTRX_3083,
	KBASE_HW_ISSUE_TTRX_3470,
	KBASE_HW_ISSUE_TTRX_3464,
	KBASE_HW_ISSUE_TITANHW_2710,
	KBASE_HW_ISSUE_GPU2022PRO_148,
	KBASE_HW_ISSUE_TITANHW_2938,
	KBASE_HW_ISSUE_END
};

__maybe_unused static const enum base_hw_issue base_hw_issues_model_tTRx[] = {
	KBASE_HW_ISSUE_5736,	       KBASE_HW_ISSUE_9435,	    KBASE_HW_ISSUE_TSIX_2033,
	KBASE_HW_ISSUE_TTRX_1337,      KBASE_HW_ISSUE_TTRX_3414,    KBASE_HW_ISSUE_TTRX_3083,
	KBASE_HW_ISSUE_TTRX_3470,      KBASE_HW_ISSUE_TTRX_3464,    KBASE_HW_ISSUE_TITANHW_2710,
	KBASE_HW_ISSUE_GPU2022PRO_148, KBASE_HW_ISSUE_TITANHW_2938, KBASE_HW_ISSUE_END
};

__maybe_unused static const enum base_hw_issue base_hw_issues_tNAx_r0p0[] = {
	KBASE_HW_ISSUE_9435,	     KBASE_HW_ISSUE_TSIX_2033,
	KBASE_HW_ISSUE_TTRX_1337,    KBASE_HW_ISSUE_TTRX_2968_TTRX_3162,
	KBASE_HW_ISSUE_TTRX_3076,    KBASE_HW_ISSUE_TTRX_921,
	KBASE_HW_ISSUE_TTRX_3414,    KBASE_HW_ISSUE_GPU2017_1336,
	KBASE_HW_ISSUE_TTRX_3083,    KBASE_HW_ISSUE_TTRX_3470,
	KBASE_HW_ISSUE_TTRX_3464,    KBASE_HW_ISSUE_TTRX_3485,
	KBASE_HW_ISSUE_TITANHW_2710, KBASE_HW_ISSUE_GPU2022PRO_148,
	KBASE_HW_ISSUE_TITANHW_2938, KBASE_HW_ISSUE_END
};

__maybe_unused static const enum base_hw_issue base_hw_issues_tNAx_r0p1[] = {
	KBASE_HW_ISSUE_9435,
	KBASE_HW_ISSUE_TSIX_2033,
	KBASE_HW_ISSUE_TTRX_1337,
	KBASE_HW_ISSUE_TTRX_2968_TTRX_3162,
	KBASE_HW_ISSUE_TTRX_3076,
	KBASE_HW_ISSUE_TTRX_921,
	KBASE_HW_ISSUE_TTRX_3414,
	KBASE_HW_ISSUE_GPU2017_1336,
	KBASE_HW_ISSUE_TTRX_3083,
	KBASE_HW_ISSUE_TTRX_3470,
	KBASE_HW_ISSUE_TTRX_3464,
	KBASE_HW_ISSUE_TITANHW_2710,
	KBASE_HW_ISSUE_GPU2022PRO_148,
	KBASE_HW_ISSUE_TITANHW_2938,
	KBASE_HW_ISSUE_END
};

__maybe_unused static const enum base_hw_issue base_hw_issues_model_tNAx[] = {
	KBASE_HW_ISSUE_5736,	       KBASE_HW_ISSUE_9435,	    KBASE_HW_ISSUE_TSIX_2033,
	KBASE_HW_ISSUE_TTRX_1337,      KBASE_HW_ISSUE_TTRX_3414,    KBASE_HW_ISSUE_TTRX_3083,
	KBASE_HW_ISSUE_TTRX_3470,      KBASE_HW_ISSUE_TTRX_3464,    KBASE_HW_ISSUE_TITANHW_2710,
	KBASE_HW_ISSUE_GPU2022PRO_148, KBASE_HW_ISSUE_TITANHW_2938, KBASE_HW_ISSUE_END
};

__maybe_unused static const enum base_hw_issue base_hw_issues_tBEx_r0p0[] = {
	KBASE_HW_ISSUE_9435,	     KBASE_HW_ISSUE_TSIX_2033,
	KBASE_HW_ISSUE_TTRX_1337,    KBASE_HW_ISSUE_TTRX_2968_TTRX_3162,
	KBASE_HW_ISSUE_TTRX_921,     KBASE_HW_ISSUE_TTRX_3414,
	KBASE_HW_ISSUE_TTRX_3083,    KBASE_HW_ISSUE_TTRX_3470,
	KBASE_HW_ISSUE_TTRX_3464,    KBASE_HW_ISSUE_TTRX_3485,
	KBASE_HW_ISSUE_TITANHW_2710, KBASE_HW_ISSUE_GPU2022PRO_148,
	KBASE_HW_ISSUE_TITANHW_2938, KBASE_HW_ISSUE_KRAKEHW_2321,
	KBASE_HW_ISSUE_END
};

__maybe_unused static const enum base_hw_issue base_hw_issues_tBEx_r0p1[] = {
	KBASE_HW_ISSUE_9435,	       KBASE_HW_ISSUE_TSIX_2033,
	KBASE_HW_ISSUE_TTRX_1337,      KBASE_HW_ISSUE_TTRX_2968_TTRX_3162,
	KBASE_HW_ISSUE_TTRX_921,       KBASE_HW_ISSUE_TTRX_3414,
	KBASE_HW_ISSUE_TTRX_3083,      KBASE_HW_ISSUE_TTRX_3470,
	KBASE_HW_ISSUE_TTRX_3464,      KBASE_HW_ISSUE_TITANHW_2710,
	KBASE_HW_ISSUE_GPU2022PRO_148, KBASE_HW_ISSUE_TITANHW_2938,
	KBASE_HW_ISSUE_KRAKEHW_2321,   KBASE_HW_ISSUE_END
};

__maybe_unused static const enum base_hw_issue base_hw_issues_tBEx_r1p0[] = {
	KBASE_HW_ISSUE_9435,	       KBASE_HW_ISSUE_TSIX_2033,
	KBASE_HW_ISSUE_TTRX_1337,      KBASE_HW_ISSUE_TTRX_2968_TTRX_3162,
	KBASE_HW_ISSUE_TTRX_921,       KBASE_HW_ISSUE_TTRX_3414,
	KBASE_HW_ISSUE_TTRX_3083,      KBASE_HW_ISSUE_TTRX_3470,
	KBASE_HW_ISSUE_TTRX_3464,      KBASE_HW_ISSUE_TITANHW_2710,
	KBASE_HW_ISSUE_GPU2022PRO_148, KBASE_HW_ISSUE_TITANHW_2938,
	KBASE_HW_ISSUE_KRAKEHW_2321,   KBASE_HW_ISSUE_END
};

__maybe_unused static const enum base_hw_issue base_hw_issues_tBEx_r1p1[] = {
	KBASE_HW_ISSUE_9435,	       KBASE_HW_ISSUE_TSIX_2033,
	KBASE_HW_ISSUE_TTRX_1337,      KBASE_HW_ISSUE_TTRX_2968_TTRX_3162,
	KBASE_HW_ISSUE_TTRX_921,       KBASE_HW_ISSUE_TTRX_3414,
	KBASE_HW_ISSUE_TTRX_3083,      KBASE_HW_ISSUE_TTRX_3470,
	KBASE_HW_ISSUE_TTRX_3464,      KBASE_HW_ISSUE_TITANHW_2710,
	KBASE_HW_ISSUE_GPU2022PRO_148, KBASE_HW_ISSUE_TITANHW_2938,
	KBASE_HW_ISSUE_KRAKEHW_2321,   KBASE_HW_ISSUE_END
};

__maybe_unused static const enum base_hw_issue base_hw_issues_model_tBEx[] = {
	KBASE_HW_ISSUE_5736,	     KBASE_HW_ISSUE_9435,
	KBASE_HW_ISSUE_TSIX_2033,    KBASE_HW_ISSUE_TTRX_1337,
	KBASE_HW_ISSUE_TTRX_3414,    KBASE_HW_ISSUE_TTRX_3083,
	KBASE_HW_ISSUE_TTRX_3470,    KBASE_HW_ISSUE_TTRX_3464,
	KBASE_HW_ISSUE_TITANHW_2710, KBASE_HW_ISSUE_GPU2022PRO_148,
	KBASE_HW_ISSUE_TITANHW_2938, KBASE_HW_ISSUE_KRAKEHW_2321,
	KBASE_HW_ISSUE_END
};

__maybe_unused static const enum base_hw_issue base_hw_issues_lBEx_r1p0[] = {
	KBASE_HW_ISSUE_9435,	     KBASE_HW_ISSUE_TSIX_2033,
	KBASE_HW_ISSUE_TTRX_1337,    KBASE_HW_ISSUE_TTRX_2968_TTRX_3162,
	KBASE_HW_ISSUE_TTRX_921,     KBASE_HW_ISSUE_TTRX_3414,
	KBASE_HW_ISSUE_TTRX_3083,    KBASE_HW_ISSUE_TTRX_3470,
	KBASE_HW_ISSUE_TTRX_3464,    KBASE_HW_ISSUE_TTRX_3485,
	KBASE_HW_ISSUE_TITANHW_2710, KBASE_HW_ISSUE_GPU2022PRO_148,
	KBASE_HW_ISSUE_TITANHW_2938, KBASE_HW_ISSUE_KRAKEHW_2321,
	KBASE_HW_ISSUE_END
};

__maybe_unused static const enum base_hw_issue base_hw_issues_lBEx_r1p1[] = {
	KBASE_HW_ISSUE_9435,	       KBASE_HW_ISSUE_TSIX_2033,
	KBASE_HW_ISSUE_TTRX_1337,      KBASE_HW_ISSUE_TTRX_2968_TTRX_3162,
	KBASE_HW_ISSUE_TTRX_921,       KBASE_HW_ISSUE_TTRX_3414,
	KBASE_HW_ISSUE_TTRX_3083,      KBASE_HW_ISSUE_TTRX_3470,
	KBASE_HW_ISSUE_TTRX_3464,      KBASE_HW_ISSUE_TITANHW_2710,
	KBASE_HW_ISSUE_GPU2022PRO_148, KBASE_HW_ISSUE_TITANHW_2938,
	KBASE_HW_ISSUE_KRAKEHW_2321,   KBASE_HW_ISSUE_END
};

__maybe_unused static const enum base_hw_issue base_hw_issues_tBAx_r0p0[] = {
	KBASE_HW_ISSUE_9435,	       KBASE_HW_ISSUE_TSIX_2033,
	KBASE_HW_ISSUE_TTRX_1337,      KBASE_HW_ISSUE_TTRX_2968_TTRX_3162,
	KBASE_HW_ISSUE_TTRX_921,       KBASE_HW_ISSUE_TTRX_3414,
	KBASE_HW_ISSUE_TTRX_3083,      KBASE_HW_ISSUE_TTRX_3470,
	KBASE_HW_ISSUE_TTRX_3464,      KBASE_HW_ISSUE_TITANHW_2710,
	KBASE_HW_ISSUE_GPU2022PRO_148, KBASE_HW_ISSUE_TITANHW_2938,
	KBASE_HW_ISSUE_KRAKEHW_2321,   KBASE_HW_ISSUE_END
};

__maybe_unused static const enum base_hw_issue base_hw_issues_tBAx_r0p1[] = {
	KBASE_HW_ISSUE_9435,	       KBASE_HW_ISSUE_TSIX_2033,
	KBASE_HW_ISSUE_TTRX_1337,      KBASE_HW_ISSUE_TTRX_2968_TTRX_3162,
	KBASE_HW_ISSUE_TTRX_921,       KBASE_HW_ISSUE_TTRX_3414,
	KBASE_HW_ISSUE_TTRX_3083,      KBASE_HW_ISSUE_TTRX_3470,
	KBASE_HW_ISSUE_TTRX_3464,      KBASE_HW_ISSUE_TITANHW_2710,
	KBASE_HW_ISSUE_GPU2022PRO_148, KBASE_HW_ISSUE_TITANHW_2938,
	KBASE_HW_ISSUE_KRAKEHW_2321,   KBASE_HW_ISSUE_END
};

__maybe_unused static const enum base_hw_issue base_hw_issues_tBAx_r0p2[] = {
	KBASE_HW_ISSUE_9435,	       KBASE_HW_ISSUE_TSIX_2033,
	KBASE_HW_ISSUE_TTRX_1337,      KBASE_HW_ISSUE_TTRX_2968_TTRX_3162,
	KBASE_HW_ISSUE_TTRX_921,       KBASE_HW_ISSUE_TTRX_3414,
	KBASE_HW_ISSUE_TTRX_3083,      KBASE_HW_ISSUE_TTRX_3470,
	KBASE_HW_ISSUE_TTRX_3464,      KBASE_HW_ISSUE_TITANHW_2710,
	KBASE_HW_ISSUE_GPU2022PRO_148, KBASE_HW_ISSUE_TITANHW_2938,
	KBASE_HW_ISSUE_KRAKEHW_2321,   KBASE_HW_ISSUE_END
};

__maybe_unused static const enum base_hw_issue base_hw_issues_model_tBAx[] = {
	KBASE_HW_ISSUE_5736,	     KBASE_HW_ISSUE_9435,
	KBASE_HW_ISSUE_TSIX_2033,    KBASE_HW_ISSUE_TTRX_1337,
	KBASE_HW_ISSUE_TTRX_3414,    KBASE_HW_ISSUE_TTRX_3083,
	KBASE_HW_ISSUE_TTRX_3470,    KBASE_HW_ISSUE_TTRX_3464,
	KBASE_HW_ISSUE_TITANHW_2710, KBASE_HW_ISSUE_GPU2022PRO_148,
	KBASE_HW_ISSUE_TITANHW_2938, KBASE_HW_ISSUE_KRAKEHW_2321,
	KBASE_HW_ISSUE_END
};

__maybe_unused static const enum base_hw_issue base_hw_issues_tODx_r0p0[] = {
	KBASE_HW_ISSUE_TSIX_2033,      KBASE_HW_ISSUE_TTRX_1337,
	KBASE_HW_ISSUE_GPU2019_3212,   KBASE_HW_ISSUE_GPU2019_3878,
	KBASE_HW_ISSUE_GPU2019_3901,   KBASE_HW_ISSUE_TITANHW_2710,
	KBASE_HW_ISSUE_GPU2022PRO_148, KBASE_HW_ISSUE_TITANHW_2938,
	KBASE_HW_ISSUE_KRAKEHW_2321,   KBASE_HW_ISSUE_END
};

__maybe_unused static const enum base_hw_issue base_hw_issues_model_tODx[] = {
	KBASE_HW_ISSUE_TSIX_2033,      KBASE_HW_ISSUE_TTRX_1337,
	KBASE_HW_ISSUE_GPU2019_3212,   KBASE_HW_ISSUE_GPU2019_3878,
	KBASE_HW_ISSUE_GPU2019_3901,   KBASE_HW_ISSUE_TITANHW_2710,
	KBASE_HW_ISSUE_GPU2022PRO_148, KBASE_HW_ISSUE_TITANHW_2938,
	KBASE_HW_ISSUE_KRAKEHW_2321,   KBASE_HW_ISSUE_END
};

__maybe_unused static const enum base_hw_issue base_hw_issues_tGRx_r0p0[] = {
	KBASE_HW_ISSUE_TSIX_2033,    KBASE_HW_ISSUE_TTRX_1337,	  KBASE_HW_ISSUE_GPU2019_3878,
	KBASE_HW_ISSUE_GPU2019_3901, KBASE_HW_ISSUE_TITANHW_2710, KBASE_HW_ISSUE_GPU2022PRO_148,
	KBASE_HW_ISSUE_TITANHW_2938, KBASE_HW_ISSUE_KRAKEHW_2321, KBASE_HW_ISSUE_END
};

__maybe_unused static const enum base_hw_issue base_hw_issues_model_tGRx[] = {
	KBASE_HW_ISSUE_TSIX_2033,    KBASE_HW_ISSUE_TTRX_1337,	  KBASE_HW_ISSUE_GPU2019_3878,
	KBASE_HW_ISSUE_GPU2019_3901, KBASE_HW_ISSUE_TITANHW_2710, KBASE_HW_ISSUE_GPU2022PRO_148,
	KBASE_HW_ISSUE_TITANHW_2938, KBASE_HW_ISSUE_KRAKEHW_2321, KBASE_HW_ISSUE_END
};

__maybe_unused static const enum base_hw_issue base_hw_issues_tVAx_r0p0[] = {
	KBASE_HW_ISSUE_TSIX_2033,    KBASE_HW_ISSUE_TTRX_1337,	  KBASE_HW_ISSUE_GPU2019_3878,
	KBASE_HW_ISSUE_GPU2019_3901, KBASE_HW_ISSUE_TITANHW_2710, KBASE_HW_ISSUE_GPU2022PRO_148,
	KBASE_HW_ISSUE_TITANHW_2938, KBASE_HW_ISSUE_KRAKEHW_2321, KBASE_HW_ISSUE_END
};

__maybe_unused static const enum base_hw_issue base_hw_issues_tVAx_r0p1[] = {
	KBASE_HW_ISSUE_TSIX_2033,    KBASE_HW_ISSUE_TTRX_1337,	  KBASE_HW_ISSUE_GPU2019_3878,
	KBASE_HW_ISSUE_GPU2019_3901, KBASE_HW_ISSUE_TITANHW_2710, KBASE_HW_ISSUE_GPU2022PRO_148,
	KBASE_HW_ISSUE_TITANHW_2938, KBASE_HW_ISSUE_KRAKEHW_2321, KBASE_HW_ISSUE_END
};

__maybe_unused static const enum base_hw_issue base_hw_issues_model_tVAx[] = {
	KBASE_HW_ISSUE_TSIX_2033,    KBASE_HW_ISSUE_TTRX_1337,	  KBASE_HW_ISSUE_GPU2019_3878,
	KBASE_HW_ISSUE_GPU2019_3901, KBASE_HW_ISSUE_TITANHW_2710, KBASE_HW_ISSUE_GPU2022PRO_148,
	KBASE_HW_ISSUE_TITANHW_2938, KBASE_HW_ISSUE_KRAKEHW_2321, KBASE_HW_ISSUE_END
};

__maybe_unused static const enum base_hw_issue base_hw_issues_tTUx_r0p0[] = {
	KBASE_HW_ISSUE_TSIX_2033,      KBASE_HW_ISSUE_TTRX_1337,    KBASE_HW_ISSUE_TURSEHW_1997,
	KBASE_HW_ISSUE_GPU2019_3878,   KBASE_HW_ISSUE_TURSEHW_2716, KBASE_HW_ISSUE_GPU2019_3901,
	KBASE_HW_ISSUE_GPU2021PRO_290, KBASE_HW_ISSUE_TITANHW_2710, KBASE_HW_ISSUE_TITANHW_2679,
	KBASE_HW_ISSUE_GPU2022PRO_148, KBASE_HW_ISSUE_TITANHW_2938, KBASE_HW_ISSUE_TURSEHW_2934,
	KBASE_HW_ISSUE_KRAKEHW_2321,   KBASE_HW_ISSUE_END
};

__maybe_unused static const enum base_hw_issue base_hw_issues_tTUx_r0p1[] = {
	KBASE_HW_ISSUE_TSIX_2033,      KBASE_HW_ISSUE_TTRX_1337,    KBASE_HW_ISSUE_TURSEHW_1997,
	KBASE_HW_ISSUE_GPU2019_3878,   KBASE_HW_ISSUE_TURSEHW_2716, KBASE_HW_ISSUE_GPU2019_3901,
	KBASE_HW_ISSUE_GPU2021PRO_290, KBASE_HW_ISSUE_TITANHW_2710, KBASE_HW_ISSUE_TITANHW_2679,
	KBASE_HW_ISSUE_GPU2022PRO_148, KBASE_HW_ISSUE_TITANHW_2922, KBASE_HW_ISSUE_TITANHW_2938,
	KBASE_HW_ISSUE_TURSEHW_2934,   KBASE_HW_ISSUE_KRAKEHW_2321, KBASE_HW_ISSUE_END
};

__maybe_unused static const enum base_hw_issue base_hw_issues_model_tTUx[] = {
	KBASE_HW_ISSUE_TSIX_2033,    KBASE_HW_ISSUE_TTRX_1337,	  KBASE_HW_ISSUE_GPU2019_3878,
	KBASE_HW_ISSUE_TURSEHW_2716, KBASE_HW_ISSUE_GPU2019_3901, KBASE_HW_ISSUE_GPU2021PRO_290,
	KBASE_HW_ISSUE_TITANHW_2710, KBASE_HW_ISSUE_TITANHW_2679, KBASE_HW_ISSUE_GPU2022PRO_148,
	KBASE_HW_ISSUE_TITANHW_2922, KBASE_HW_ISSUE_TITANHW_2938, KBASE_HW_ISSUE_TURSEHW_2934,
	KBASE_HW_ISSUE_KRAKEHW_2321, KBASE_HW_ISSUE_END
};

__maybe_unused static const enum base_hw_issue base_hw_issues_tTUx_r1p0[] = {
	KBASE_HW_ISSUE_TSIX_2033,    KBASE_HW_ISSUE_TTRX_1337,	  KBASE_HW_ISSUE_GPU2019_3878,
	KBASE_HW_ISSUE_TURSEHW_2716, KBASE_HW_ISSUE_GPU2019_3901, KBASE_HW_ISSUE_GPU2021PRO_290,
	KBASE_HW_ISSUE_TITANHW_2710, KBASE_HW_ISSUE_TITANHW_2679, KBASE_HW_ISSUE_GPU2022PRO_148,
	KBASE_HW_ISSUE_TITANHW_2922, KBASE_HW_ISSUE_TITANHW_2938, KBASE_HW_ISSUE_TURSEHW_2934,
	KBASE_HW_ISSUE_KRAKEHW_2321, KBASE_HW_ISSUE_END
};

__maybe_unused static const enum base_hw_issue base_hw_issues_tTUx_r1p1[] = {
	KBASE_HW_ISSUE_TSIX_2033,    KBASE_HW_ISSUE_TTRX_1337,	  KBASE_HW_ISSUE_GPU2019_3878,
	KBASE_HW_ISSUE_TURSEHW_2716, KBASE_HW_ISSUE_GPU2019_3901, KBASE_HW_ISSUE_GPU2021PRO_290,
	KBASE_HW_ISSUE_TITANHW_2710, KBASE_HW_ISSUE_TITANHW_2679, KBASE_HW_ISSUE_GPU2022PRO_148,
	KBASE_HW_ISSUE_TITANHW_2922, KBASE_HW_ISSUE_TITANHW_2938, KBASE_HW_ISSUE_TURSEHW_2934,
	KBASE_HW_ISSUE_KRAKEHW_2321, KBASE_HW_ISSUE_END
};

__maybe_unused static const enum base_hw_issue base_hw_issues_tTUx_r1p2[] = {
	KBASE_HW_ISSUE_TSIX_2033,    KBASE_HW_ISSUE_TTRX_1337,	  KBASE_HW_ISSUE_GPU2019_3878,
	KBASE_HW_ISSUE_TURSEHW_2716, KBASE_HW_ISSUE_GPU2019_3901, KBASE_HW_ISSUE_GPU2021PRO_290,
	KBASE_HW_ISSUE_TITANHW_2710, KBASE_HW_ISSUE_TITANHW_2679, KBASE_HW_ISSUE_GPU2022PRO_148,
	KBASE_HW_ISSUE_TITANHW_2922, KBASE_HW_ISSUE_TITANHW_2938, KBASE_HW_ISSUE_TURSEHW_2934,
	KBASE_HW_ISSUE_KRAKEHW_2321, KBASE_HW_ISSUE_END
};

__maybe_unused static const enum base_hw_issue base_hw_issues_tTUx_r1p3[] = {
	KBASE_HW_ISSUE_TSIX_2033,    KBASE_HW_ISSUE_TTRX_1337,	  KBASE_HW_ISSUE_GPU2019_3878,
	KBASE_HW_ISSUE_TURSEHW_2716, KBASE_HW_ISSUE_GPU2019_3901, KBASE_HW_ISSUE_GPU2021PRO_290,
	KBASE_HW_ISSUE_TITANHW_2710, KBASE_HW_ISSUE_TITANHW_2679, KBASE_HW_ISSUE_GPU2022PRO_148,
	KBASE_HW_ISSUE_TITANHW_2922, KBASE_HW_ISSUE_TITANHW_2938, KBASE_HW_ISSUE_TURSEHW_2934,
	KBASE_HW_ISSUE_KRAKEHW_2321, KBASE_HW_ISSUE_END
};

__maybe_unused static const enum base_hw_issue base_hw_issues_model_tTIx[] = {
	KBASE_HW_ISSUE_TSIX_2033,
	KBASE_HW_ISSUE_TTRX_1337,
	KBASE_HW_ISSUE_TURSEHW_2716,
	KBASE_HW_ISSUE_GPU2021PRO_290,
	KBASE_HW_ISSUE_TITANHW_2710,
	KBASE_HW_ISSUE_TITANHW_2679,
	KBASE_HW_ISSUE_GPU2022PRO_148,
	KBASE_HW_ISSUE_TITANHW_2922,
	KBASE_HW_ISSUE_TITANHW_2952,
	KBASE_HW_ISSUE_TITANHW_2938,
	KBASE_HW_ISSUE_TURSEHW_2934,
	KBASE_HW_ISSUE_KRAKEHW_2321,
	KBASE_HW_ISSUE_END
};

__maybe_unused static const enum base_hw_issue base_hw_issues_tTIx_r0p0[] = {
	KBASE_HW_ISSUE_TSIX_2033,
	KBASE_HW_ISSUE_TTRX_1337,
	KBASE_HW_ISSUE_TURSEHW_2716,
	KBASE_HW_ISSUE_GPU2021PRO_290,
	KBASE_HW_ISSUE_TITANHW_2710,
	KBASE_HW_ISSUE_TITANHW_2679,
	KBASE_HW_ISSUE_GPU2022PRO_148,
	KBASE_HW_ISSUE_TITANHW_2922,
	KBASE_HW_ISSUE_TITANHW_2952,
	KBASE_HW_ISSUE_TITANHW_2938,
	KBASE_HW_ISSUE_TURSEHW_2934,
	KBASE_HW_ISSUE_KRAKEHW_2321,
	KBASE_HW_ISSUE_END
};

__maybe_unused static const enum base_hw_issue base_hw_issues_tTIx_r0p1[] = {
	KBASE_HW_ISSUE_TSIX_2033,      KBASE_HW_ISSUE_TTRX_1337,    KBASE_HW_ISSUE_TURSEHW_2716,
	KBASE_HW_ISSUE_GPU2021PRO_290, KBASE_HW_ISSUE_TITANHW_2710, KBASE_HW_ISSUE_TITANHW_2679,
	KBASE_HW_ISSUE_GPU2022PRO_148, KBASE_HW_ISSUE_TITANHW_2938, KBASE_HW_ISSUE_TURSEHW_2934,
	KBASE_HW_ISSUE_KRAKEHW_2321,   KBASE_HW_ISSUE_END
};

__maybe_unused static const enum base_hw_issue base_hw_issues_tKRx_r0p0[] = {
	KBASE_HW_ISSUE_TTRX_1337,    KBASE_HW_ISSUE_TURSEHW_2716, KBASE_HW_ISSUE_GPU2022PRO_148,
	KBASE_HW_ISSUE_KRAKEHW_2151, KBASE_HW_ISSUE_KRAKEHW_2269, KBASE_HW_ISSUE_TITANHW_2922,
	KBASE_HW_ISSUE_TURSEHW_2934, KBASE_HW_ISSUE_KRAKEHW_2321, KBASE_HW_ISSUE_END
};

__maybe_unused static const enum base_hw_issue base_hw_issues_tKRx_r0p1[] = {
	KBASE_HW_ISSUE_TTRX_1337,    KBASE_HW_ISSUE_TURSEHW_2716, KBASE_HW_ISSUE_GPU2022PRO_148,
	KBASE_HW_ISSUE_KRAKEHW_2151, KBASE_HW_ISSUE_KRAKEHW_2269, KBASE_HW_ISSUE_TURSEHW_2934,
	KBASE_HW_ISSUE_KRAKEHW_2321, KBASE_HW_ISSUE_END
};

__maybe_unused static const enum base_hw_issue base_hw_issues_model_tKRx[] = {
	KBASE_HW_ISSUE_TTRX_1337,    KBASE_HW_ISSUE_TURSEHW_2716, KBASE_HW_ISSUE_GPU2022PRO_148,
	KBASE_HW_ISSUE_KRAKEHW_2151, KBASE_HW_ISSUE_KRAKEHW_2269, KBASE_HW_ISSUE_TURSEHW_2934,
	KBASE_HW_ISSUE_KRAKEHW_2321, KBASE_HW_ISSUE_END
};


#endif /* _KBASE_HWCONFIG_ISSUES_H_ */
